
/*  @(#)help.c 1.3 95/02/06
 *
 *  Various routines associated with the help popup and help text.
 *
 *  Copyright (c) Rich Burridge.    All rights reserved.
 *
 *  Permission is given to distribute these sources, as long as the
 *  copyright messages are not removed, and no monies are exchanged.
 *
 *  No responsibility is taken for any errors or inaccuracies inherent
 *  either to the comments or the code of this program, but if
 *  reported to me then an attempt will be made to fix them.
 */

#include "mptool.h"
#include <Xm/Text.h>
#include <Xm/MessageB.h>

Widget help_popup ;           /* Help popup. */
Widget *helpwarr ;

static void add_help_text      (char *) ;
static void do_show_help       (void) ;
static void help_done          (Widget) ;


void
add_help(Widget widget, String text[])
{
  XtAddCallback(widget, XmNhelpCallback, (XtCallbackProc) do_help, text) ;
}


static void
add_help_text(char *text)
{
  static char *buf = "\n\n" ;
  int nbytes ;
  XmTextPosition pos ;
 
  pos = XmTextGetLastPosition(helpwarr[WI_HELP_TEXT]) ;
  nbytes = strlen(text) ;
  XmTextInsert(helpwarr[WI_HELP_TEXT], pos, text) ;
  XmTextInsert(helpwarr[WI_HELP_TEXT], pos+nbytes, buf) ;
}


void
do_help(Widget parent, char **text, XmAnyCallbackStruct *s)
{
  char buf[BUFFERSIZE], *p ;
  int i ;
  Widget dialog ;
  XmString str ;
 
  p = buf ;
  for (i = 0; text[i]; i++)
    {
      p += strlen(strcpy(p, text[i])) ;
      *p++ = '\n' ;
      *p = 0 ;
    }
  str = XmStringCreateLtoR(buf, XmSTRING_DEFAULT_CHARSET) ;
  dialog = XmCreateInformationDialog(parent, "help", NULL, 0) ;
  XtVaSetValues(dialog,
                XmNmessageString, str,
                XmNautoUnmanage,  FALSE,
                0) ;
  XtVaSetValues(XtParent(dialog), XmNtitle, "Help", 0) ;
  XmStringFree(str) ;

  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON)) ;
  XtSetSensitive(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON), False) ;
  XtAddCallback(dialog, XmNokCallback, (XtCallbackProc) help_done, NULL) ;
  XtManageChild(dialog) ;
  XtPopup(XtParent(dialog), XtGrabNone) ;
}


void
do_help_dismiss(void)
{
  XtUnmapWidget(help_popup) ;
  XtUnmanageChild(help_popup) ;
}


void
do_help_general(void)
{
  do_show_help() ;
  add_help_text(concat_strs(wi_main_frame_help)) ;
  add_help_text(concat_strs(wi_filename_help)) ;
  add_help_text(concat_strs(wi_file_type_help)) ;
  add_help_text(concat_strs(wi_output_help)) ;
  add_help_text(concat_strs(wi_org_type_help)) ;
  add_help_text(concat_strs(wi_paper_size_help)) ;
  add_help_text(concat_strs(wi_subject_help)) ;
  add_help_text(concat_strs(wi_no_sides_help)) ;
  add_help_text(concat_strs(wi_copies_help)) ;
  add_help_text(concat_strs(wi_print_help)) ;
  add_help_text(concat_strs(wi_reset_help)) ;
  show_popup(help_popup) ;
}


void
do_help_headers(void)
{
  do_show_help() ;
  add_help_text(concat_strs(wi_header_frame_help)) ;
  add_help_text(concat_strs(wi_headers_all_help)) ;
  add_help_text(concat_strs(wi_hdr_list_help)) ;
  add_help_text(concat_strs(wi_hdr_add_help)) ;
  add_help_text(concat_strs(wi_hdr_remove_help)) ;
  add_help_text(concat_strs(wi_hdr_apply_help)) ;
  add_help_text(concat_strs(wi_hdr_reset_help)) ;
  add_help_text(concat_strs(wi_hdr_dismiss_help)) ;
  show_popup(help_popup) ;
}


void
do_help_input(void)
{
  do_show_help() ;
  add_help_text(concat_strs(wi_input_frame_help)) ;
  add_help_text(concat_strs(wi_prologue_help)) ;
  add_help_text(concat_strs(wi_opt_folder_help)) ;
  add_help_text(concat_strs(wi_opt_digest_help)) ;
  add_help_text(concat_strs(wi_props_elm_help)) ;
  add_help_text(concat_strs(wi_props_from_help)) ;
  add_help_text(concat_strs(wi_props_content_help)) ;
  add_help_text(concat_strs(wi_props_modtime_help)) ;
  add_help_text(concat_strs(wi_input_apply_help)) ;
  add_help_text(concat_strs(wi_input_reset_help)) ;
  add_help_text(concat_strs(wi_input_dismiss_help)) ;
  show_popup(help_popup) ;
}


void
do_help_output(void)
{
  do_show_help() ;
  add_help_text(concat_strs(wi_output_frame_help)) ;
  add_help_text(concat_strs(wi_printer_name_help)) ;
  add_help_text(concat_strs(wi_line_length_help)) ;
  add_help_text(concat_strs(wi_page_length_help)) ;
  add_help_text(concat_strs(wi_output_bsd_help)) ;
  add_help_text(concat_strs(wi_output_apply_help)) ;
  add_help_text(concat_strs(wi_output_reset_help)) ;
  add_help_text(concat_strs(wi_output_dismiss_help)) ;
  show_popup(help_popup) ;
}


void
do_help_printing(void)
{
  do_show_help() ;
  add_help_text(concat_strs(wi_props_frame_help)) ;
  add_help_text(concat_strs(wi_props_col_help)) ;
  add_help_text(concat_strs(wi_props_tab_help)) ;
  add_help_text(concat_strs(wi_opt_header_help)) ;
  add_help_text(concat_strs(wi_props_wrap_help)) ;
  add_help_text(concat_strs(wi_props_flip_help)) ;
  add_help_text(concat_strs(wi_props_forcepage_help)) ;
  add_help_text(concat_strs(wi_props_number_help)) ;
  add_help_text(concat_strs(wi_props_reverse_help)) ;
  add_help_text(concat_strs(wi_props_apply_help)) ;
  add_help_text(concat_strs(wi_props_reset_help)) ;
  add_help_text(concat_strs(wi_props_dismiss_help)) ;
  show_popup(help_popup) ;
}


void
do_help_save(void)
{
  do_show_help() ;
  add_help_text(concat_strs(wi_props_defs_help)) ;
  show_popup(help_popup) ;
}


void
do_help_username(void)
{
  do_show_help() ;
  add_help_text(concat_strs(wi_un_form_help)) ;
  add_help_text(concat_strs(wi_aliasname_help)) ;
  add_help_text(concat_strs(wi_props_passc_help)) ;
  add_help_text(concat_strs(wi_props_passw_help)) ;
  add_help_text(concat_strs(wi_un_apply_help)) ;
  add_help_text(concat_strs(wi_un_reset_help)) ;
  add_help_text(concat_strs(wi_un_dismiss_help)) ;
  show_popup(help_popup) ;
}


static void
do_show_help(void)
{
  if (!help_popup)
    {
      help_popup = tu_help_frame_widget("help", warr[WI_MAIN_FRAME],
                                           &helpwarr) ;
      add_delete_callback(help_popup) ;
      add_help(helpwarr[WI_HELP_FORM],    wi_help_text_help) ;
      add_help(helpwarr[WI_HELP_TEXT],    wi_help_text_help) ;
      add_help(helpwarr[WI_HELP_DISMISS], wi_help_dismiss_help) ;
    }
  XmTextSetString(helpwarr[WI_HELP_TEXT], "") ;
}


void
Help(void)
{
  helpwarr = NULL ;        /* No widgets defined initially. */
  help_popup = NULL ;      /* No help popup window initially. */
}


static void
help_done(Widget dialog)
{
  XtDestroyWidget(dialog) ;
}
